#include <iostream>
#include <fstream>
#include "framebuffer.h"
#include "m3x3.h"

#include <tiffio.h>

using namespace std;

FrameBuffer::FrameBuffer(int u0, int v0, 
  int _w, int _h) : Fl_Gl_Window(u0, v0, _w, _h, 0) {

  w = _w;
  h = _h;
  pix = new unsigned int[w*h];
  Set(0xFFFF0000);

}

void FrameBuffer::Set(unsigned int color) {

  for (int i = 0; i < w*h; i++)
    pix[i] = color;

}

void FrameBuffer::draw() {

  glDrawPixels(w, h, GL_RGBA, 
    GL_UNSIGNED_BYTE, pix);

}


void FrameBuffer::Set(int u, int v, unsigned int color) {

  pix[(h-1-v)*w+u] = color;

}

void FrameBuffer::SetChecker(int cw, unsigned int colors[2]) {

  for (int v = 0; v < h; v++) {
    int cv = v / cw;
    for (int u = 0; u < w; u++) {
      int cu = u / cw;
      unsigned int color = ((cu+cv)%2) ? colors[1] : colors[0];
      Set(u, v, color);
    }
  }

}

void FrameBuffer::SetGuarded(float uf, 
                             float vf, unsigned int color) {

  if (uf < 0.0f || uf >= (float)w 
    || vf < 0.0f || vf >= (float)h)
    return;

  Set((int)uf, (int)vf, color);

}

void FrameBuffer::EdgeMap(FrameBuffer *&edgeMap) {

  edgeMap = new FrameBuffer(w, 0, w, h);
  edgeMap->label("Edge map");

  edgeMap->Set(0xFF000000);

  M3x3 krn;
  krn[0] = V3(-1.0f, -2.0f, -1.0f);
  krn[1] = V3(-2.0f, 12.0f, -2.0f);
  krn[2] = V3(-1.0f, -2.0f, -1.0f);

  for (int v = 1; v < h-1; v++) {
    for (int u = 1; u < w-1; u++) {
      
    }
  }

}

int FrameBuffer::Load(char *tiffFileName) {
  
  if (!strcmp(tiffFileName + strlen(tiffFileName)-4, ".tif")) {
    
    ifstream tst(tiffFileName);
    if (tst.fail()) {
      cerr << "INFO: cannot open: " << tiffFileName << endl;
      return 0;
    }
    tst.close();
    
    TIFF* tif = TIFFOpen(tiffFileName, "r");
    
    if (tif) {
      if (pix)
        delete pix;
      
      cerr << "INFO: started loading: " << tiffFileName << endl;
      
      TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &w);
      TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &h);
      pix = new unsigned int[w*h];
      
      if (!TIFFReadRGBAImage(tif, w, h, (uint32*) pix, 0)) {
        delete pix;
        pix = 0;
        w = h = 0;
        return 0;
      }
      
      TIFFClose(tif);
      label(tiffFileName);
      resize(0, 0, w, h);
      cerr << "INFO: done loading: " << tiffFileName << endl;
      return 1;
    }
    return 0;
  }
  cerr << "INFO: do not know how to open *" << tiffFileName + strlen(tiffFileName)-4 << endl;
  return 0;
}

FrameBuffer::FrameBuffer() : w(0), h(0), pix(0), Fl_Gl_Window(0, 0, 0, 0, 0)  {}

